import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import { Tree } from './utils/Tree';
import _isActive from './utils/isActive';
import { matchPattern } from './PatternUtils';
import warning from './routerWarning';
import createMemoryHistory from './createMemoryHistory';

var checkIntegrity = function checkIntegrity(nodes) {
  return !nodes.some(function (node, index) {
    return node.level !== index + 1;
  });
};

var createParams = function createParams(_ref) {
  var paramValues = _ref.paramValues,
      paramNames = _ref.paramNames;
  return paramNames.reduce(function (acc, name, index) {
    var _objectSpread2;

    return _objectSpread({}, acc, (_objectSpread2 = {}, _objectSpread2[name] = paramValues[index], _objectSpread2));
  }, {});
};

var findRedirect = function findRedirect(nodes) {
  var tail = nodes[nodes.length - 1];

  if (tail.redirect) {
    return tail;
  }

  return null;
};

var createComponents =
/*#__PURE__*/
function () {
  var _ref3 = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee(_ref2) {
    var nodes, routeInfo, makePromise, indexRoute, components;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            nodes = _ref2.nodes, routeInfo = _ref2.routeInfo;

            makePromise = function makePromise(getComponent) {
              return new Promise(function (resolve, reject) {
                getComponent(routeInfo, function (err, component) {
                  if (err) return reject(err);
                  return resolve(component);
                });
              });
            };

            indexRoute = nodes.findIndex(function (node) {
              return node.fromIndex;
            });

            if (indexRoute !== -1 && indexRoute === nodes.length - 1) {
              nodes = [].concat(nodes, [nodes[indexRoute].indexNode]);
            }

            _context.next = 6;
            return Promise.all(nodes.filter(function (node) {
              return node.component || node.getComponent;
            }).map(function (node) {
              return node.component ? Promise.resolve(node.component) : makePromise(node.getComponent);
            }));

          case 6:
            components = _context.sent;
            return _context.abrupt("return", components);

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function createComponents(_x) {
    return _ref3.apply(this, arguments);
  };
}(); // From tree to Array


var matchRoutes =
/*#__PURE__*/
function () {
  var _ref4 = _asyncToGenerator(
  /*#__PURE__*/
  _regeneratorRuntime.mark(function _callee2(tree, location, remainingPathname) {
    var match, redirectNode, nodes, paramValues, paramNames, params, routeInfo, components;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (remainingPathname === undefined) {
              if (location.pathname.charAt(0) !== '/') {
                location = _objectSpread({}, location, {
                  pathname: "/" + location.pathname
                });
              }

              remainingPathname = location.pathname;
            } // Tree.tap(tree)


            match = Tree.reduce(function (acc, node) {
              var _acc = acc,
                  remainingPathname = _acc.remainingPathname,
                  paramNames = _acc.paramNames,
                  paramValues = _acc.paramValues;
              var pattern = node.path || '';
              if (acc.isFinished) return acc;
              if (node.index) return acc;
              acc = _objectSpread({}, acc, {
                nodes: acc.nodes.filter(function (n) {
                  return n.level < node.level;
                })
              }); // console.log(node.level, acc)

              if (pattern.charAt(0) === '/') {
                remainingPathname = location.pathname;
                paramNames = [];
                paramValues = [];
              }

              if (pattern === '') {
                return _objectSpread({}, acc, {
                  nodes: [].concat(acc.nodes, [node])
                });
              }

              var matched = matchPattern(pattern, remainingPathname);

              if (matched) {
                acc = {
                  remainingPathname: matched.remainingPathname,
                  paramNames: [].concat(paramNames, matched.paramNames),
                  paramValues: [].concat(paramValues, matched.paramValues),
                  nodes: [].concat(acc.nodes, [node])
                };
              }

              if ((matched === null || matched === void 0 ? void 0 : matched.remainingPathname) === '') {
                acc = _objectSpread({}, acc, {
                  isFinished: checkIntegrity(acc.nodes)
                });
              }

              return acc;
            }, {
              remainingPathname: remainingPathname,
              paramNames: [],
              paramValues: [],
              nodes: [],
              isFinished: false
            }, tree);
            redirectNode = findRedirect(match.nodes);

            if (!redirectNode) {
              _context2.next = 5;
              break;
            }

            return _context2.abrupt("return", {
              redirectLocation: redirectNode.to
            });

          case 5:
            nodes = match.nodes, paramValues = match.paramValues, paramNames = match.paramNames;
            params = createParams({
              paramNames: paramNames,
              paramValues: paramValues
            });
            routeInfo = {
              location: location,
              params: params,
              routes: nodes
            };
            _context2.next = 10;
            return createComponents({
              nodes: nodes,
              routeInfo: routeInfo
            });

          case 10:
            components = _context2.sent;
            return _context2.abrupt("return", {
              routeInfo: routeInfo,
              components: components
            });

          case 12:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function matchRoutes(_x2, _x3, _x4) {
    return _ref4.apply(this, arguments);
  };
}();

export var createTransitionManager = function createTransitionManager(_ref5) {
  var externalHistory = _ref5.history,
      jsonRoutes = _ref5.jsonRoutes;
  var history = externalHistory || createMemoryHistory();
  var state = {// components,
    // router,
    // matchContext: {transitionManager, router},
    // location,
    // params,
    // routes
  };
  return {
    match: function () {
      var _match = _asyncToGenerator(
      /*#__PURE__*/
      _regeneratorRuntime.mark(function _callee3(location) {
        var _ref6, redirectLocation, routeInfo, components;

        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                location = location ? history.createLocation(location) : history.getCurrentLocation();
                _context3.next = 3;
                return matchRoutes(jsonRoutes, location);

              case 3:
                _ref6 = _context3.sent;
                redirectLocation = _ref6.redirectLocation;
                routeInfo = _ref6.routeInfo;
                components = _ref6.components;
                state = _objectSpread({}, state, {}, routeInfo, {
                  components: components,
                  location: location
                });
                return _context3.abrupt("return", {
                  redirectLocation: redirectLocation,
                  routeInfo: routeInfo,
                  components: components
                });

              case 9:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3);
      }));

      function match(_x5) {
        return _match.apply(this, arguments);
      }

      return match;
    }(),
    listen: function listen(listener) {
      var historyListener =
      /*#__PURE__*/
      function () {
        var _ref7 = _asyncToGenerator(
        /*#__PURE__*/
        _regeneratorRuntime.mark(function _callee4(location) {
          var _ref8, redirectLocation, _routeInfo, components;

          return _regeneratorRuntime.wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  if (!(state.location === location)) {
                    _context4.next = 4;
                    break;
                  }

                  listener(null, {
                    params: state.params,
                    components: state.components
                  });
                  _context4.next = 21;
                  break;

                case 4:
                  _context4.prev = 4;
                  _context4.next = 7;
                  return matchRoutes(jsonRoutes, location);

                case 7:
                  _ref8 = _context4.sent;
                  redirectLocation = _ref8.redirectLocation;
                  _routeInfo = _ref8.routeInfo;
                  components = _ref8.components;

                  if (!redirectLocation) {
                    _context4.next = 13;
                    break;
                  }

                  return _context4.abrupt("return", history.replace(redirectLocation));

                case 13:
                  if (!components) {
                    warning(false, 'Location "%s" did not match any routes', location.pathname + location.search + location.hash);
                  }

                  state = _objectSpread({}, state, {}, _routeInfo, {
                    components: components,
                    location: location
                  });
                  return _context4.abrupt("return", listener(null, state));

                case 18:
                  _context4.prev = 18;
                  _context4.t0 = _context4["catch"](4);
                  listener(_context4.t0);

                case 21:
                case "end":
                  return _context4.stop();
              }
            }
          }, _callee4, null, [[4, 18]]);
        }));

        return function historyListener(_x6) {
          return _ref7.apply(this, arguments);
        };
      }();

      var unsubscribe = history.listen(historyListener);

      if (state.location) {
        // Picking up on a matchContext.
        listener(null, state);
      } else {
        historyListener(history.getCurrentLocation());
      }

      return unsubscribe;
    },
    isActive: function isActive(location, indexOnly) {
      location = history.createLocation(location);
      return _isActive(location, indexOnly, state.location, state.routes, state.params);
    }
  };
};